import csv
import json
import os

import avro.schema
from avro.datafile import DataFileReader, DataFileWriter
from avro.io import DatumReader, DatumWriter
from avro.io import Validate as validate


class AvroHelper:

    @staticmethod
    def parse_schema(schema_path):
        with open(schema_path, 'r') as data:
            return avro.schema.Parse(data.read())

    @staticmethod
    def get_fields_from_schema(path):
        with open(path, 'rU') as data:
            content = json.loads(data.read())
            header_fields = []
            for field in content["fields"]:
                header_fields.append(field["name"])
            return header_fields

    def read_avro_file(self, avro_file_path, schema_path, consumer_target_filepath, data_format):
        fields = AvroHelper.get_fields_from_schema(schema_path)
        schema = self.parse_schema(schema_path)
        reader = DataFileReader(open(avro_file_path, "rb"), DatumReader())
        target_file = open(consumer_target_filepath, 'w')
        writer = csv.DictWriter(target_file, fieldnames=fields)
        writer.writeheader()
        value = []
        for datum in reader:
            if data_format == "json":
                value.append(json.dumps(datum))
            else:
                value.append(datum)
            vResult = validate(schema, datum)
            if vResult:
                writer.writerow(datum)
            else:
                raise ValueError("Schema Validation Failed")

        reader.close()
        target_file.close()
        os.remove(consumer_target_filepath)
        os.remove(avro_file_path)
        return value

    def generate_avro_data(self, records, schema_path, outpath_path, message_format):
        schema = self.parse_schema(schema_path)
        with open(outpath_path, 'wb') as out:
            writer = DataFileWriter(out, DatumWriter(), schema)
            if message_format == "json":
                writer.append(records)
                writer.close()
            else:
                for record in records:
                    writer.append(record)
                else:
                    writer.close()
