import importlib

import pandas as pd
from pandas import DataFrame


class TextHelper:

    @staticmethod
    def read_json(json_message, dtypes):
        data_frame = pd.read_json(json_message, dtype=dtypes)
        return data_frame

    @staticmethod
    def build_dataframe_from_json(json_message, dtypes, date_columns):
        if json_message == '"[]"':
            return DataFrame()
        data_frame = pd.read_json(json_message, dtype=dtypes, convert_dates=date_columns)
        return data_frame

    @staticmethod
    def convert_date_to_monthbegin(data_frame, date_column_name):
        data_frame[date_column_name] = data_frame[date_column_name] + pd.offsets.MonthEnd(n=0) - pd.offsets.MonthBegin(
            n=1)
        return data_frame

    @staticmethod
    def get_dtypes(dtypes_module_path):
        dtypes = importlib.import_module(dtypes_module_path).dtypes
        return dtypes

    @staticmethod
    def get_pipeline_dtypes(pipeline_name):
        dtypes = importlib.import_module(f'tasks.{pipeline_name}.{pipeline_name}_dtype').dtypes
        return dtypes

    @staticmethod
    def get_pipeline_datecolumns(pipeline_name):
        date_columns = importlib.import_module(f'tasks.{pipeline_name}.{pipeline_name}_dtype').date_columns
        return date_columns

    @staticmethod
    def convert_filter_values_to_list(filter_values):
        filter_condition_values = []
        for value in filter_values.split(';'):
            try:
                filter_condition_values.append(int(value))
            except ValueError:
                try:
                    filter_condition_values.append(float(value))
                except ValueError:
                    filter_condition_values.append(value)
        return filter_condition_values
