from kafka import KafkaConsumer, TopicPartition
from pdp_pipeline_core.core import logger


class PdpKafkaConsumer:

    def get_consumer(self):
        return self._consumer

    def __init__(self, bootstrap_servers, group_id):
        self._consumer = KafkaConsumer(bootstrap_servers=bootstrap_servers, group_id=group_id,
                                       auto_offset_reset='latest', enable_auto_commit=False,
                                       consumer_timeout_ms=1000)

    def consume(self, topic):
        # self._consumer.subscribe(topic)
        partition = TopicPartition(topic, 0)
        self._consumer.assign([partition])
        consumer = self._consumer
        seeking_offset = consumer.committed(partition)
        if seeking_offset:
            consumer.seek(partition, seeking_offset)
            logger.info("seeking offset {0}".format(seeking_offset))
        else:
            logger.info("seeking first message")
            consumer.seek(partition, 0)
        # Reading from offset 10
        # consumer.seek(partition, 10)
        # self._consumer.seek_to_beginning()
        while True:
            try:
                record = self._consumer.poll(timeout_ms=500, max_records=20)
                for t, message in record.items():
                    msg = message[0]
                    message = {"offset": msg.offset, "partition": msg.partition, "timestamp": msg.timestamp,
                               "topic": msg.topic, "headers": msg.headers,
                               "value": message[0].value}
                    yield message

            except Exception as e:
                logger.error("Message failed {}".format(e.args))
                continue
