import json
import io
import json
import os
import time
from datetime import datetime

import pandas as pd
from kafka import KafkaProducer

from pdp_pipeline_core.core import  logger, exception_logger

from pdp_pipeline_core.Utils.avro_helper import AvroHelper


class PdpKafkaProducer:

    def __init__(self, bootstrap_servers):
        self._producer = KafkaProducer(bootstrap_servers=bootstrap_servers)

    @staticmethod
    def read_csv(path):
        df = pd.read_csv(path)
        df = df.where((pd.notnull(df)), None)
        input_records = df.to_dict(orient='records')
        for row in input_records:
            print(row)
            yield row

    @staticmethod
    def get_avro_data(records, record_name, schema_path, message_format):
        timestamp = str(datetime.timestamp(datetime.now()))
        output_path = '{0}_{1}.avro'.format(timestamp, record_name)
        AvroHelper().generate_avro_data(records, schema_path, output_path, message_format)
        avro_file = open(output_path, "rb")
        data = avro_file.read()
        bytes_writer = io.BytesIO()
        # Write the Confluent "Magic Byte"
        bytes_writer.write(bytes([0]))
        bytes_writer.write(data)
        bytes_writer.write(
            int.to_bytes(1, 4, byteorder='big'))
        data = bytes_writer.getvalue()
        avro_file.close()
        os.remove(output_path)
        return data

    def get_data(self, input_file, record_name, schema_path, message_format):
        if message_format == "csv":
            records = self.read_csv()
            return self.get_avro_data(records, record_name, schema_path, message_format)
        else:
            if message_format == 'json':
                with open(input_file, 'r') as input_data_file:
                    records = json.loads(input_data_file.read())
                return self.get_avro_data(records, record_name, schema_path, message_format)

    @staticmethod
    def on_send_success(record_metadata):
        logger.info("successfully published!")
        logger.info(record_metadata.topic)
        logger.info(record_metadata.partition)
        logger.info(record_metadata.offset)

    @staticmethod
    def on_send_error(exception):
        exception_logger.exception(exception.args)

    # def publish(self, topic, input_file, record_name, schema_path, message_format):
    #     data = self.get_data(input_file, record_name, schema_path, message_format)
    #     self._producer.send(topic, data, partition=0).add_callback(self.on_send_success).add_errback(self.on_send_error)
    #     time.sleep(1)
    #     self._producer.close()
    #
    #     os.remove(input_file)

    def publish(self, topic, data, header):
        self._producer.send(topic, data, headers=header, partition=0).add_callback(
            self.on_send_success).add_errback(self.on_send_error)
        time.sleep(1)
        self._producer.close()
