import json

from pdp_pipeline_core.core import logger
from pdp_pipeline_core.core.log_decorator import logit
from pdp_pipeline_core.repository.connection_manager import create_engine, get_session


@logit
def bulk_insert(session,  data_frame, table_class, search_path):
    session.execute(f"SET search_path TO {search_path}")
    bulk_insert_dict = json.loads(data_frame.to_json(orient='records', date_format='iso'))
    session.bulk_insert_mappings(
        table_class,
        bulk_insert_dict,
    )
    logger.info("Data Inserted")


@logit
def execute_query(session, query):
    session.execute(query)
    logger.info(f"Query Executed: {query}")

@logit
def delete_and_save_all(aurora_connection_string, data, table_name, schema):
    data_engine = create_engine(aurora_connection_string)
    session = get_session(data_engine)
    path = f'SET search_path TO public,"{schema}"'
    session.execute(path)
    query = f'TRUNCATE TABLE "{schema}".{table_name} RESTART IDENTITY CASCADE'
    execute_query(session, query)
    session.add_all(data)
    session.commit()
